@@ -20,4 +20,16 @@ if os.name == "posix":
 			
 		def __del__(self):
 			self.handle.close()
+		
+		def __enter__ (self):
+			self.acquire ()
+			return self
+	
+		def __exit__ (self, type, value, tb):
+			self.release ()
+
+else:
+	import multiprocessing
+	
+	Lock = multiprocessing.RLock ()
 	
\ No newline at end of file
